// SPDX-License-Identifier: LGPL-3.0-or-later
// Copyright © 2020-2021 Florian Schmaus
#pragma once

#include <atomic>	 // for atomic

#include "BlockablePurpose.hpp"
#include "PrivateSemaphore.hpp"	 // for PrivateSemaphore

class Context;
namespace emper::io {
class Future;
}

class BinaryPrivateSemaphore final : public PrivateSemaphore {
 private:
	enum State {
		initial,
		signaled,
		blocked,
	};

	std::atomic<State> bpsState;

	// It would be OK to not initialize this member, since it is only
	// set after the State is set accordingly and then only read if
	// this state is found. However clang-tidy then starts to complain
	// about clang-analyzer-optin.cplusplus.UninitializedObject, and
	// furthermore, all call sites of wait() yield a
	// clang-analyzer-optin.cplusplus.VirtualCall (it appears
	// clang-tidy only checks if the object is fully initialized, not
	// if the vcall is actually performed in a constructor.
	Context* blockedContext = nullptr;

	BinaryPrivateSemaphore(emper::BlockablePurpose blockablePurpose)
			: PrivateSemaphore(blockablePurpose), bpsState(initial) {}

 protected:
	auto signalInternal() -> Context* override;

 public:
	BinaryPrivateSemaphore() : BinaryPrivateSemaphore(emper::BlockablePurpose::GENERIC) {}

	~BinaryPrivateSemaphore() override = default;

	void wait() override;

	void reset() { bpsState = initial; };

	// Workaround for doxygen choking on the following friend declaration with
	// Parsing file emper/emper/BinaryPrivateSemaphore.hpp:54: error: no uniquely matching class
	// member found for
	//   Future
	//   (warning treated as error, aborting now)
	//! @cond Doxygen_Suppress
	friend emper::io::Future;
	//! @endcond
};

using BPS = BinaryPrivateSemaphore;
