// SPDX-License-Identifier: LGPL-3.0-or-later
// Copyright © 2020 Florian Schmaus
#pragma once

#include "Common.hpp"					// for func_t
#include "Debug.hpp"					// for LogSubsystem, LogSubsystem::CM, Logger
#include "MemoryManager.hpp"	// for MemoryManager

class Context;
class Runtime;

#define CONTEXT_MANAGER_FIRST_LAYER_QUEUE_SIZE 64

class ContextManager
		: public Logger<LogSubsystem::CM>,
			protected MemoryManager<Context, 128, CONTEXT_MANAGER_FIRST_LAYER_QUEUE_SIZE> {
 private:
	const Runtime& runtime;

	friend Context;

 public:
	ContextManager(Runtime& runtime);

	auto getFreeContext() -> Context*;

	static void putFreeContext(Context* context);

	void start();

	void saveAndStartNew(func_t freshContextHook);

	[[noreturn]] void discardAndResume(Context* context);
};
