// SPDX-License-Identifier: LGPL-3.0-or-later
// Copyright © 2020 Florian Schmaus
#pragma once

#include <cstddef>

namespace adt {

template <typename T, size_t N>
class BoundedBumpArray {
 private:
	static_assert(N > 1, "BoundedBumpArray size N must be greater than 1");

	size_t nextFreeElement = 0;

	// NOLINTNEXTLINE(modernize-avoid-c-arrays)
	T* array[N];

 public:
	auto isFull() -> bool { return nextFreeElement + 1 == N; }

	auto isEmpty() -> bool { return nextFreeElement == 0; }

	auto put(T* t) -> bool {
		if (isFull()) return false;

		array[nextFreeElement++] = t;
		return true;
	}

	auto get() -> T* {
		if (isEmpty()) return nullptr;

		return array[--nextFreeElement];
	}
};

}	 // namespace adt
