// SPDX-License-Identifier: LGPL-3.0-or-later
// Copyright © 2021 Florian Fischer
#pragma once

#include <string>

#include "Debug.hpp"

namespace emper::lib::env {

static auto getBoolFromEnv(const std::string&& key) -> std::optional<bool> {
	DBG("parse " << key << " environment variable");
	char* envVar = std::getenv(key.c_str());
	if (!envVar) {
		return std::nullopt;
	}

	std::string envStr(envVar);
	if (envStr == "true") {
		return true;
	}

	if (envStr == "false") {
		return false;
	}

	DIE_MSG(key << " has invalid value: " << envStr << " (expected true or false)");
}

}	 // namespace emper::lib::env
