// SPDX-License-Identifier: LGPL-3.0-or-later
// Copyright © 2021 Florian Schmaus
#pragma once

#include <cstddef>
#include <cstdint>
#include <iostream>
#include <vector>

#include "BlockablePurpose.hpp"
#include "emper-common.h"

enum class LogSubsystem;
template <LogSubsystem logSubsystem>
class Blockable;	// IWYU pragma: keep
class Runtime;

namespace emper::stats {

class Worker {
 private:
	size_t maxBlockedContexts = 0;

	uint32_t blockedGeneric = 0;
	std::vector<uint32_t> unblockAffinitiesGeneric;

	uint32_t blockedIo = 0;
	std::vector<uint32_t> unblockAffinitiesIo;

	Worker(workerid_t workerCount);

	void recordBlockedContext(size_t blockedContextCount, BlockablePurpose blockablePurpose);

	void recordUnblockedContext(workeraffinity_t workerAffinity, BlockablePurpose blockablePurpose);

	auto operator+=(const Worker& other) -> Worker&;

 public:
	void print(std::ostream& out = std::cout);

	template <LogSubsystem>
	friend class ::Blockable;
	friend class ::Runtime;
};

extern thread_local Worker* worker;

}	 // namespace emper::stats
